@@ -43,13 +43,14 @@ typedef struct
     GlJournalEntry *entry;
     GlUtilClockFormat clock_format;
     GtkListBox *entries_box;
+    GtkSizeGroup *category_sizegroup;
+    GtkSizeGroup *message_sizegroup;
+    GtkSizeGroup *time_sizegroup;
     GtkWidget *categories;
     GtkWidget *event_search;
     GtkWidget *event_scrolled;
     GtkWidget *search_entry;
     gchar *search_text;
-
-    GlEventViewRowStyle current_row_style;
 } GlEventViewListPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GlEventViewList, gl_event_view_list, GTK_TYPE_BOX)
@@ -122,6 +123,29 @@ utf8_strcasestr (const gchar *potential_hit,
   return matches;
 }
 
+static void
+listbox_update_header_func (GtkListBoxRow *row,
+                            GtkListBoxRow *before,
+                            gpointer user_data)
+{
+    GtkWidget *current;
+
+    if (before == NULL)
+    {
+        gtk_list_box_row_set_header (row, NULL);
+        return;
+    }
+
+    current = gtk_list_box_row_get_header (row);
+
+    if (current == NULL)
+    {
+        current = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
+        gtk_widget_show (current);
+        gtk_list_box_row_set_header (row, current);
+    }
+}
+
 static gboolean
 listbox_search_filter_func (GtkListBoxRow *row,
                             GlEventViewList *view)
@@ -305,11 +329,46 @@ static GtkWidget *
 gl_event_list_view_create_row_widget (gpointer item,
                                       gpointer user_data)
 {
+    GtkWidget *rtn;
+    GtkWidget *message_label;
+    GtkWidget *time_label;
+    GlCategoryList *list;
+    GlCategoryListFilter filter;
     GlEventViewList *view = user_data;
 
     GlEventViewListPrivate *priv = gl_event_view_list_get_instance_private (view);
 
-    return gl_event_view_row_new (item, priv->current_row_style, priv->clock_format);
+    list = GL_CATEGORY_LIST (priv->categories);
+    filter = gl_category_list_get_category (list);
+
+    if (filter == GL_CATEGORY_LIST_FILTER_IMPORTANT)
+    {
+        GtkWidget *category_label;
+
+        rtn = gl_event_view_row_new (item,
+                                     priv->clock_format,
+                                     GL_EVENT_VIEW_ROW_CATEGORY_IMPORTANT);
+
+        category_label = gl_event_view_row_get_category_label (GL_EVENT_VIEW_ROW (rtn));
+        gtk_size_group_add_widget (GTK_SIZE_GROUP (priv->category_sizegroup),
+                                   category_label);
+    }
+    else
+    {
+        rtn = gl_event_view_row_new (item,
+                                     priv->clock_format,
+                                     GL_EVENT_VIEW_ROW_CATEGORY_NONE);
+    }
+
+    message_label = gl_event_view_row_get_message_label (GL_EVENT_VIEW_ROW (rtn));
+    time_label = gl_event_view_row_get_time_label (GL_EVENT_VIEW_ROW (rtn));
+
+    gtk_size_group_add_widget (GTK_SIZE_GROUP (priv->message_sizegroup),
+                               message_label);
+    gtk_size_group_add_widget (GTK_SIZE_GROUP (priv->time_sizegroup),
+                               time_label);
+
+    return rtn;
 }
 
 static gchar *
@@ -351,7 +410,6 @@ on_notify_category (GlCategoryList *list,
               /* Alert or emergency priority. */
               const gchar * query[] = { "PRIORITY=0", "PRIORITY=1", "PRIORITY=2", "PRIORITY=3", NULL };
 
-              priv->current_row_style = GL_EVENT_VIEW_ROW_STYLE_CMDLINE;
               gl_journal_model_set_matches (priv->journal_model, query);
             }
             break;
@@ -360,7 +418,6 @@ on_notify_category (GlCategoryList *list,
             {
                 const gchar *query[] = { NULL };
 
-                priv->current_row_style = GL_EVENT_VIEW_ROW_STYLE_CMDLINE;
                 gl_journal_model_set_matches (priv->journal_model, query);
             }
             break;
@@ -376,7 +433,6 @@ on_notify_category (GlCategoryList *list,
                                          NULL,
                                          NULL };
 
-                priv->current_row_style = GL_EVENT_VIEW_ROW_STYLE_CMDLINE;
                 uid_str = create_uid_match_string ();
                 query[3] = uid_str;
                 gl_journal_model_set_matches (priv->journal_model, query);
@@ -389,7 +445,6 @@ on_notify_category (GlCategoryList *list,
             {
                 const gchar *query[] = { "_TRANSPORT=kernel", NULL };
 
-                priv->current_row_style = GL_EVENT_VIEW_ROW_STYLE_SIMPLE;
                 gl_journal_model_set_matches (priv->journal_model, query);
             }
             break;
@@ -398,7 +453,6 @@ on_notify_category (GlCategoryList *list,
             {
                 const gchar *query[] = { "_TRANSPORT=kernel", "_KERNEL_DEVICE", NULL };
 
-                priv->current_row_style = GL_EVENT_VIEW_ROW_STYLE_SIMPLE;
                 gl_journal_model_set_matches (priv->journal_model, query);
             }
             break;
@@ -407,7 +461,6 @@ on_notify_category (GlCategoryList *list,
             {
                 const gchar *query[] = { "_AUDIT_SESSION", NULL };
 
-                priv->current_row_style = GL_EVENT_VIEW_ROW_STYLE_CMDLINE;
                 gl_journal_model_set_matches (priv->journal_model, query);
             }
             break;
@@ -567,6 +620,9 @@ gl_event_view_list_finalize (GObject *object)
 
     g_clear_object (&priv->journal_model);
     g_clear_pointer (&priv->search_text, g_free);
+    g_object_unref (priv->category_sizegroup);
+    g_object_unref (priv->message_sizegroup);
+    g_object_unref (priv->time_sizegroup);
 }
 
 static void
@@ -607,6 +663,9 @@ gl_event_view_list_init (GlEventViewList *view)
 
     priv = gl_event_view_list_get_instance_private (view);
     priv->search_text = NULL;
+    priv->category_sizegroup = gtk_size_group_new (GTK_SIZE_GROUP_BOTH);
+    priv->message_sizegroup = gtk_size_group_new (GTK_SIZE_GROUP_BOTH);
+    priv->time_sizegroup = gtk_size_group_new (GTK_SIZE_GROUP_BOTH);
     categories = GL_CATEGORY_LIST (priv->categories);
 
     priv->journal_model = gl_journal_model_new ();
@@ -620,6 +679,9 @@ gl_event_view_list_init (GlEventViewList *view)
                              gl_event_list_view_create_row_widget,
                              view, NULL);
 
+    gtk_list_box_set_header_func (GTK_LIST_BOX (priv->entries_box),
+                                  (GtkListBoxUpdateHeaderFunc) listbox_update_header_func,
+                                  NULL, NULL);
     gtk_list_box_set_filter_func (GTK_LIST_BOX (priv->entries_box),
                                   (GtkListBoxFilterFunc) listbox_search_filter_func,
                                   view, NULL);
