@@ -27,9 +27,9 @@
 enum
 {
     PROP_0,
+    PROP_CATEGORY,
     PROP_CLOCK_FORMAT,
     PROP_ENTRY,
-    PROP_STYLE,
     N_PROPERTIES
 };
 
@@ -41,15 +41,48 @@ struct _GlEventViewRow
 
 typedef struct
 {
+    GlEventViewRowCategory category;
     GlUtilClockFormat clock_format;
     GlJournalEntry *entry;
-    GlEventViewRowStyle style;
+    GtkWidget *category_label;
+    GtkWidget *message_label;
+    GtkWidget *time_label;
 } GlEventViewRowPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GlEventViewRow, gl_event_view_row, GTK_TYPE_LIST_BOX_ROW)
 
 static GParamSpec *obj_properties[N_PROPERTIES] = { NULL, };
 
+GtkWidget *
+gl_event_view_row_get_category_label (GlEventViewRow *row)
+{
+    GlEventViewRowPrivate *priv;
+
+    priv = gl_event_view_row_get_instance_private (row);
+
+    return priv->category_label;
+}
+
+GtkWidget *
+gl_event_view_row_get_message_label (GlEventViewRow *row)
+{
+    GlEventViewRowPrivate *priv;
+
+    priv = gl_event_view_row_get_instance_private (row);
+
+    return priv->message_label;
+}
+
+GtkWidget *
+gl_event_view_row_get_time_label (GlEventViewRow *row)
+{
+    GlEventViewRowPrivate *priv;
+
+    priv = gl_event_view_row_get_instance_private (row);
+
+    return priv->time_label;
+}
+
 static void
 gl_event_view_row_finalize (GObject *object)
 {
@@ -70,15 +103,15 @@ gl_event_view_row_get_property (GObject *object,
 
     switch (prop_id)
     {
+        case PROP_CATEGORY:
+            g_value_set_enum (value, priv->category);
+            break;
         case PROP_CLOCK_FORMAT:
             g_value_set_enum (value, priv->clock_format);
             break;
         case PROP_ENTRY:
             g_value_set_object (value, priv->entry);
             break;
-        case PROP_STYLE:
-            g_value_set_enum (value, priv->style);
-            break;
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
             break;
@@ -96,15 +129,15 @@ gl_event_view_row_set_property (GObject *object,
 
     switch (prop_id)
     {
+        case PROP_CATEGORY:
+            priv->category = g_value_get_enum (value);
+            break;
         case PROP_CLOCK_FORMAT:
             priv->clock_format = g_value_get_enum (value);
             break;
         case PROP_ENTRY:
             priv->entry = g_value_dup_object (value);
             break;
-        case PROP_STYLE:
-            priv->style = g_value_get_enum (value);
-            break;
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
             break;
@@ -112,142 +145,115 @@ gl_event_view_row_set_property (GObject *object,
 }
 
 static void
-gl_event_view_row_create_cmdline (GlEventViewRow *row)
+gl_event_view_row_construct_category_label (GlEventViewRow *row,
+                                            GlJournalEntry *entry)
 {
+    gint uid;
     GlEventViewRowPrivate *priv;
-    GtkStyleContext *context;
-    GtkWidget *grid;
-    gchar *markup;
-    GtkWidget *label;
-    gchar *time;
-    gboolean rtl;
-    GtkWidget *image;
-    GlJournalEntry *entry;
-    GDateTime *now;
 
+    uid = gl_util_get_uid ();
     priv = gl_event_view_row_get_instance_private (row);
-    entry = priv->entry;
 
-    rtl = (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL);
-
-    context = gtk_widget_get_style_context (GTK_WIDGET (row));
-    gtk_style_context_add_class (context, "event");
-    grid = gtk_grid_new ();
-    gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
-    gtk_container_add (GTK_CONTAINER (row), grid);
-
-    if (gl_journal_entry_get_command_line (entry))
-      markup = g_markup_printf_escaped ("<b>%s</b>", gl_journal_entry_get_command_line (entry));
+    /* The priority given to the categories should be determined by how
+     * specific the checks are. The applications category is the most
+     * specific, followed by the hardware category, then kernel, security
+     * and finally the least-specific other category. So we check the category
+     * in the order of applications, hardware, system, security and other. */
+    if ((g_strcmp0 (gl_journal_entry_get_transport (entry), "kernel") == 0
+         || g_strcmp0 (gl_journal_entry_get_transport (entry), "stdout") == 0
+         || g_strcmp0 (gl_journal_entry_get_transport (entry), "syslog") == 0)
+        && gl_journal_entry_get_uid (entry) == uid)
+    {
+        priv->category_label = gtk_label_new (_("Applications"));
+    }
+    else if (g_strcmp0 (gl_journal_entry_get_transport (entry), "kernel") == 0
+             && gl_journal_entry_get_kernel_device (entry) != NULL)
+    {
+        priv->category_label = gtk_label_new (_("Hardware"));
+    }
+    else if (g_strcmp0 (gl_journal_entry_get_transport (entry), "kernel") == 0)
+    {
+        priv->category_label = gtk_label_new (_("System"));
+    }
+    else if (gl_journal_entry_get_audit_session (entry) != NULL)
+    {
+        priv->category_label = gtk_label_new (_("Security"));
+    }
     else
-      markup = g_strdup ("");
-
-    label = gtk_label_new (NULL);
-    gtk_widget_set_direction (label, GTK_TEXT_DIR_LTR);
-    gtk_widget_set_hexpand (label, TRUE);
-    gtk_widget_set_halign (label, GTK_ALIGN_START);
-    gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
-    gtk_label_set_markup (GTK_LABEL (label), markup);
-    g_free (markup);
-    gtk_grid_attach (GTK_GRID (grid), label, rtl ? 1 : 0, 0, 1, 1);
-
-    label = gtk_label_new (gl_journal_entry_get_message (entry));
-    gtk_widget_set_direction (label, GTK_TEXT_DIR_LTR);
-    context = gtk_widget_get_style_context (GTK_WIDGET (label));
-    gtk_style_context_add_class (context, "event-monospace");
-    gtk_widget_set_halign (label, GTK_ALIGN_START);
-    gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
-    gtk_grid_attach (GTK_GRID (grid), label, 0, 1, 2, 1);
-
-    now = g_date_time_new_now_local ();
-    time = gl_util_timestamp_to_display (gl_journal_entry_get_timestamp (entry),
-                                         now, priv->clock_format);
-    g_date_time_unref (now);
-    label = gtk_label_new (time);
-    context = gtk_widget_get_style_context (GTK_WIDGET (label));
-    gtk_style_context_add_class (context, "dim-label");
-    gtk_style_context_add_class (context, "event-time");
-    gtk_widget_set_halign (label, GTK_ALIGN_END);
-    gtk_grid_attach (GTK_GRID (grid), label, rtl ? 0 : 1, 0, 1, 1);
-
-    image = gtk_image_new_from_icon_name ("go-next-symbolic",
-                                          GTK_ICON_SIZE_MENU);
-    gtk_grid_attach (GTK_GRID (grid), image, 2, 0, 1, 2);
-
-    g_free (time);
-}
-
-static void
-gl_event_view_row_create_simple (GlEventViewRow *row)
-{
-    GlEventViewRowPrivate *priv;
-    GtkStyleContext *context;
-    GtkWidget *grid;
-    GtkWidget *label;
-    gchar *time;
-    gboolean rtl;
-    GtkWidget *image;
-    GlJournalEntry *entry;
-    GDateTime *now;
-
-    priv = gl_event_view_row_get_instance_private (row);
-    entry = priv->entry;
-
-    rtl = (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL);
-
-    context = gtk_widget_get_style_context (GTK_WIDGET (row));
-    gtk_style_context_add_class (context, "event");
-    grid = gtk_grid_new ();
-    gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
-    gtk_container_add (GTK_CONTAINER (row), grid);
-
-    label = gtk_label_new (gl_journal_entry_get_message (entry));
-    gtk_widget_set_direction (label, GTK_TEXT_DIR_LTR);
-    context = gtk_widget_get_style_context (GTK_WIDGET (label));
-    gtk_style_context_add_class (context, "event-monospace");
-    gtk_widget_set_hexpand (label, TRUE);
-    gtk_widget_set_halign (label, GTK_ALIGN_START);
-    gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
-    gtk_grid_attach (GTK_GRID (grid), label, 0, 1, 1, 1);
-
-    now = g_date_time_new_now_local ();
-    time = gl_util_timestamp_to_display (gl_journal_entry_get_timestamp (entry),
-                                         now, priv->clock_format);
-    g_date_time_unref (now);
-    label = gtk_label_new (time);
-    context = gtk_widget_get_style_context (GTK_WIDGET (label));
-    gtk_style_context_add_class (context, "dim-label");
-    gtk_style_context_add_class (context, "event-time");
-    gtk_widget_set_halign (label, GTK_ALIGN_END);
-    gtk_grid_attach (GTK_GRID (grid), label, rtl ? 1 : 0, 0, 1, 1);
-
-    image = gtk_image_new_from_icon_name ("go-next-symbolic",
-                                          GTK_ICON_SIZE_MENU);
-    gtk_grid_attach (GTK_GRID (grid), image, 1, 0, 1, 2);
-
-    g_free (time);
+    {
+        priv->category_label = gtk_label_new (_("Other"));
+    }
 }
 
 static void
 gl_event_view_row_constructed (GObject *object)
 {
+    GtkStyleContext *context;
+    GtkWidget *grid;
+    gchar *time;
+    gboolean rtl;
+    GlEventViewRowCategory category;
+    GlUtilClockFormat tmp_clock_format;
+    GlJournalEntry *tmp_entry;
+    GlJournalEntry *entry;
+    GDateTime *now;
     GlEventViewRow *row = GL_EVENT_VIEW_ROW (object);
     GlEventViewRowPrivate *priv;
 
     priv = gl_event_view_row_get_instance_private (row);
+    entry = priv->entry;
 
-    /* contruct-only properties have already been set. */
-    switch (priv->style)
+    rtl = (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL);
+
+    context = gtk_widget_get_style_context (GTK_WIDGET (row));
+    gtk_style_context_add_class (context, "event");
+    grid = gtk_grid_new ();
+    gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
+    gtk_container_add (GTK_CONTAINER (row), grid);
+
+    g_object_get (object,
+                  "category", &category,
+                  "clock-format", &tmp_clock_format,
+                  "entry", &tmp_entry,
+                  NULL);
+
+    if (category == GL_EVENT_VIEW_ROW_CATEGORY_IMPORTANT)
     {
-        case GL_EVENT_VIEW_ROW_STYLE_CMDLINE:
-            gl_event_view_row_create_cmdline (row);
-            break;
-        case GL_EVENT_VIEW_ROW_STYLE_SIMPLE:
-            gl_event_view_row_create_simple (row);
-            break;
-        default:
-            g_assert_not_reached ();
+        gl_event_view_row_construct_category_label (row, entry);
+
+        context = gtk_widget_get_style_context (GTK_WIDGET (priv->category_label));
+        gtk_style_context_add_class (context, "dim-label");
+        gtk_label_set_xalign (GTK_LABEL (priv->category_label), 0);
+        gtk_grid_attach (GTK_GRID (grid), priv->category_label,
+                         rtl ? 2 : 0, 0, 1, 1);
     }
 
+    priv->message_label = gtk_label_new (gl_journal_entry_get_message (entry));
+    gtk_widget_set_direction (priv->message_label, GTK_TEXT_DIR_LTR);
+    context = gtk_widget_get_style_context (GTK_WIDGET (priv->message_label));
+    gtk_style_context_add_class (context, "event-monospace");
+    gtk_widget_set_halign (priv->message_label, GTK_ALIGN_START);
+    gtk_label_set_ellipsize (GTK_LABEL (priv->message_label),
+                             PANGO_ELLIPSIZE_END);
+    gtk_label_set_xalign (GTK_LABEL (priv->message_label), 0);
+    gtk_grid_attach (GTK_GRID (grid), priv->message_label,
+                     1, 0, 1, 1);
+
+    now = g_date_time_new_now_local ();
+    time = gl_util_timestamp_to_display (gl_journal_entry_get_timestamp (entry),
+                                         now, priv->clock_format);
+    g_date_time_unref (now);
+    priv->time_label = gtk_label_new (time);
+    context = gtk_widget_get_style_context (GTK_WIDGET (priv->time_label));
+    gtk_style_context_add_class (context, "dim-label");
+    gtk_style_context_add_class (context, "event-time");
+    gtk_widget_set_halign (priv->time_label, GTK_ALIGN_END);
+    gtk_label_set_xalign (GTK_LABEL (priv->time_label), 1);
+    gtk_grid_attach (GTK_GRID (grid), priv->time_label, rtl ? 0 : 2, 0, 1, 1);
+
+    g_free (time);
+    g_object_unref (tmp_entry);
+
     G_OBJECT_CLASS (gl_event_view_row_parent_class)->constructed (object);
 }
 
@@ -261,6 +267,14 @@ gl_event_view_row_class_init (GlEventViewRowClass *klass)
     gobject_class->get_property = gl_event_view_row_get_property;
     gobject_class->set_property = gl_event_view_row_set_property;
 
+    obj_properties[PROP_CATEGORY] = g_param_spec_enum ("category", "Category",
+                                                       "Filter rows from important category",
+                                                       GL_TYPE_EVENT_VIEW_ROW_CATEGORY,
+                                                       GL_EVENT_VIEW_ROW_CATEGORY_NONE,
+                                                       G_PARAM_READWRITE |
+                                                       G_PARAM_CONSTRUCT_ONLY |
+                                                       G_PARAM_STATIC_STRINGS);
+
     obj_properties[PROP_CLOCK_FORMAT] = g_param_spec_enum ("clock-format", "Clock format",
                                                            "Format of the clock in which to show timestamps",
                                                            GL_TYPE_UTIL_CLOCK_FORMAT,
@@ -276,14 +290,6 @@ gl_event_view_row_class_init (GlEventViewRowClass *klass)
                                                       G_PARAM_CONSTRUCT_ONLY |
                                                       G_PARAM_STATIC_STRINGS);
 
-    obj_properties[PROP_STYLE] = g_param_spec_enum ("style", "Style",
-                                                    "Row display style",
-                                                    GL_TYPE_EVENT_VIEW_ROW_STYLE,
-                                                    GL_EVENT_VIEW_ROW_STYLE_CMDLINE,
-                                                    G_PARAM_READWRITE |
-                                                    G_PARAM_CONSTRUCT_ONLY |
-                                                    G_PARAM_STATIC_STRINGS);
-
     g_object_class_install_properties (gobject_class, N_PROPERTIES,
                                        obj_properties);
 }
@@ -291,7 +297,8 @@ gl_event_view_row_class_init (GlEventViewRowClass *klass)
 static void
 gl_event_view_row_init (GlEventViewRow *row)
 {
-    /* See gl_event_view_row_constructed (). */
+    /* The widgets are initialized in gl_event_view_row_constructed (), because
+     * at _init() time the construct-only properties have not been set. */
 }
 
 GlJournalEntry *
@@ -308,9 +315,10 @@ gl_event_view_row_get_entry (GlEventViewRow *row)
 
 GtkWidget *
 gl_event_view_row_new (GlJournalEntry *entry,
-                       GlEventViewRowStyle style,
-                       GlUtilClockFormat clock_format)
+                       GlUtilClockFormat clock_format,
+                       GlEventViewRowCategory category)
 {
-    return g_object_new (GL_TYPE_EVENT_VIEW_ROW, "style", style, "entry",
-                         entry, "clock-format", clock_format, NULL);
+    return g_object_new (GL_TYPE_EVENT_VIEW_ROW, "entry", entry,
+                         "clock-format", clock_format,
+                         "category", category, NULL);
 }
